<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\Admin;

use Locr\WordPress\Plugin\LocrMaps\Admin\SettingsPage\{General, Icons, OnlineMaps};
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\SettingsMessageId;
use Locr\WordPress\Plugin\LocrMaps\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the settings page.
 */
final class SettingsPage {
	public const MAIN_PAGE                  = Plugin::SLUG;
	private const ERROR_SLUG_TITLE          = Plugin::SLUG . '_options';
	public const ERROR_SLUG_NAME_ICONS      = Plugin::SLUG . '_icons';
	public const ERROR_SLUG_NAME_NONCE      = Plugin::SLUG . '_nonce';
	public const ERROR_SLUG_NAME_ONLINEMAPS = Plugin::SLUG . '_onlinemaps';

	/**
	 * This function initializes the settings page.
	 */
	public static function init(): void {
		add_action( 'admin_menu', array( self::class, 'add_menu' ) );
		add_action( 'admin_init', array( self::class, 'init_settings' ) );
		add_action( 'admin_notices', array( self::class, 'handle_messages' ) );
	}

	/**
	 * This function adds the main menu.
	 */
	public static function add_menu(): void {
		add_menu_page(
			page_title: 'locrMAPS',
			menu_title: 'locrMAPS',
			capability: 'manage_options',
			menu_slug: self::MAIN_PAGE,
			callback: array( General::class, 'render_settings_page' ),
			icon_url: 'dashicons-location-alt',
			position: 59
		);

		General::add_submenu();
		Icons::add_submenu();
		OnlineMaps::add_submenu();

		do_action( 'locrmaps_after_add_menu' );
	}

	/**
	 * This function handles the messages.
	 */
	public static function handle_messages(): void {
		$page = (string) ( filter_input( INPUT_GET, 'page', FILTER_DEFAULT ) ?? '' );
		if (
			self::MAIN_PAGE !== $page &&
			General::SUB_PAGE_NAME !== $page &&
			Icons::SUB_PAGE_NAME !== $page &&
			OnlineMaps::SUB_PAGE_NAME !== $page
		) {
			return;
		}

		$message = (string) ( filter_input( INPUT_GET, 'message', FILTER_DEFAULT ) ?? '' );
		if ( empty( $message ) ) {
			return;
		}
		$settings_message_id = SettingsMessageId::tryFrom( $message );
		if ( is_null( $settings_message_id ) ) {
			add_settings_error(
				setting: self::ERROR_SLUG_TITLE,
				code: self::ERROR_SLUG_TITLE,
				message: __( 'Unhandled message', 'locrmaps-textdomain' ),
				type: 'notice'
			);
			return;
		}

		switch ( $settings_message_id ) {
			case SettingsMessageId::IconDeleted:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ICONS,
					message: __( 'The icon was deleted.', 'locrmaps-textdomain' ),
					type: 'success'
				);
				break;

			case SettingsMessageId::IconsDeleted:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ICONS,
					message: __( 'The selected icons were deleted.', 'locrmaps-textdomain' ),
					type: 'success'
				);
				break;

			case SettingsMessageId::IconNotDeleted:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ICONS,
					message: __( 'The icon could not be deleted.', 'locrmaps-textdomain' ),
					type: 'error'
				);
				break;

			case SettingsMessageId::IconsNotDeleted:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ICONS,
					message: __( 'The selected icons could not been deleted.', 'locrmaps-textdomain' ),
					type: 'error'
				);
				break;

			case SettingsMessageId::IdsIsNotAnArray:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ONLINEMAPS,
					message: __( 'ids are required to be an array.', 'locrmaps-textdomain' ),
					type: 'error'
				);
				break;

			case SettingsMessageId::InvalidNonce:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_NONCE,
					message: __( 'The nonce is invalid.', 'locrmaps-textdomain' ),
					type: 'error'
				);
				break;

			case SettingsMessageId::NoNonce:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_NONCE,
					message: __( 'The nonce is missing.', 'locrmaps-textdomain' ),
					type: 'error'
				);
				break;

			case SettingsMessageId::OnlineMapDeleted:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ONLINEMAPS,
					message: __( 'The selected ONLINEmap was deleted.', 'locrmaps-textdomain' ),
					type: 'success'
				);
				break;

			case SettingsMessageId::OnlineMapLocationDeleted:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ONLINEMAPS,
					message: __( 'The selected ONLINEmap location was deleted.', 'locrmaps-textdomain' ),
					type: 'success'
				);
				break;

			case SettingsMessageId::OnlineMapLocationNotDeleted:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ONLINEMAPS,
					message: __( 'The selected ONLINEmap location was not deleted.', 'locrmaps-textdomain' ),
					type: 'error'
				);
				break;

			case SettingsMessageId::OnlineMapLocationsDeleted:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ONLINEMAPS,
					message: __( 'The selected ONLINEmap locations were deleted.', 'locrmaps-textdomain' ),
					type: 'success'
				);
				break;

			case SettingsMessageId::OnlineMapLocationsNotDeleted:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ONLINEMAPS,
					message: __( 'The selected ONLINEmap locations were not deleted.', 'locrmaps-textdomain' ),
					type: 'error'
				);
				break;

			case SettingsMessageId::OnlineMapNotDeleted:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ONLINEMAPS,
					message: __( 'The selected ONLINEmap was not deleted.', 'locrmaps-textdomain' ),
					type: 'error'
				);
				break;

			case SettingsMessageId::OnlineMapNotFound:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ONLINEMAPS,
					message: __( 'The selected ONLINEmap was not found.', 'locrmaps-textdomain' ),
					type: 'error'
				);
				break;

			case SettingsMessageId::OnlineMapsDeleted:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ONLINEMAPS,
					message: __( 'The selected ONLINEmaps were deleted.', 'locrmaps-textdomain' ),
					type: 'success'
				);
				break;

			case SettingsMessageId::OnlineMapsNotDeleted:
				add_settings_error(
					setting: self::ERROR_SLUG_TITLE,
					code: self::ERROR_SLUG_NAME_ONLINEMAPS,
					message: __( 'The ONLINEmaps could not been deleted.', 'locrmaps-textdomain' ),
					type: 'error'
				);
				break;

			default:
				break;
		}
	}

	/**
	 * This function initializes the settings.
	 */
	public static function init_settings(): void {
		General::register_settings();
		Icons::register_settings();
		Icons::init();
		OnlineMaps::init();
	}
}
