<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\Entities;

use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\Icon\{Anchor, Id, Name, Scale, Size, Url};

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for an icon.
 *
 * @property-read Anchor $anchor The anchor.
 * @property-read Id $id The id.
 * @property-read Name $name The name.
 * @property-read Scale $scale The scale.
 * @property-read Size $size The size.
 * @property-read Url $url The url.
 */
final class Icon {
	/**
	 * The constructor.
	 *
	 * @param Id     $id   The id.
	 * @param Url    $url  The url.
	 * @param Size   $size The size.
	 * @param Name   $name The name.
	 * @param Scale  $scale The scale.
	 * @param Anchor $anchor The anchor.
	 */
	public function __construct(
		private Id $id,
		private Url $url,
		private Size $size,
		private Name $name = new Name( '' ),
		private Scale $scale = new Scale( 1.0 ),
		private Anchor $anchor = Anchor::Center,
	) {
		if ( '' === $this->name->value ) {
			$parsed_url = wp_parse_url( (string) $url->value );
			if ( is_array( $parsed_url ) && isset( $parsed_url['path'] ) ) {
				$pathinfo   = pathinfo( $parsed_url['path'] );
				$this->name = new Name( $pathinfo['filename'] );
			}
		}
		if ( '' === $this->name->value ) {
			$this->name = new Name( md5( $url->value ) );
		}
	}

	/**
	 * The magick getter.
	 *
	 * @param string $name The name of the property.
	 * @return mixed
	 * @throws \InvalidArgumentException If the property does not exist.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'anchor' => $this->anchor,
			'id' => $this->id,
			'name' => $this->name,
			'scale' => $this->scale,
			'size' => $this->size,
			'url' => $this->url,
			default => throw new \InvalidArgumentException( 'The property "' . esc_html( $name ) . '" does not exist.' ),
		};
	}

	/**
	 * Builds an icon instance from a database row.
	 *
	 * @param mixed $row The database row.
	 * @throws \InvalidArgumentException If the "id"-field is missing.
	 */
	public static function from_db_row( mixed $row ): self {
		if ( ! is_array( $row ) ) {
			throw new \InvalidArgumentException( 'The row must be an array.' );
		}
		if ( ! isset( $row['id'] ) ) {
			throw new \InvalidArgumentException( 'The "id"-field is missing from the database.' );
		}
		// name can be null!
		if ( ! array_key_exists( 'name', $row ) ) {
			throw new \InvalidArgumentException( 'The "name"-field is missing from the database.' );
		}
		if ( ! isset( $row['icon_url'] ) ) {
			throw new \InvalidArgumentException( 'The "icon_url"-field is missing from the database.' );
		}
		if ( ! isset( $row['width'] ) ) {
			throw new \InvalidArgumentException( 'The "width"-field is missing from the database.' );
		}
		if ( ! isset( $row['height'] ) ) {
			throw new \InvalidArgumentException( 'The "height"-field is missing from the database.' );
		}
		if ( ! isset( $row['scale'] ) ) {
			throw new \InvalidArgumentException( 'The "scale"-field is missing from the database.' );
		}
		if ( ! isset( $row['anchor'] ) ) {
			throw new \InvalidArgumentException( 'The "anchor"-field is missing from the database.' );
		}
		if ( ! is_string( $row['anchor'] ) && ! is_int( $row['anchor'] ) ) {
			throw new \InvalidArgumentException( 'The "anchor"-field must be a string or an integer.' );
		}

		return new self(
			id: new Id( $row['id'] ),
			name: new Name( $row['name'] ?? '' ),
			url: new Url( $row['icon_url'] ),
			size: new Size( $row['width'], $row['height'] ),
			scale: new Scale( $row['scale'] ),
			anchor: Anchor::from( $row['anchor'] ),
		);
	}

	/**
	 * Converts the icon to an array.
	 *
	 * @return array<mixed>
	 */
	public function to_array(): array {
		return array(
			'id'     => $this->id->value,
			'name'   => (string) $this->name,
			'scale'  => $this->scale->value,
			'size'   => $this->size->to_array(),
			'url'    => (string) $this->url,
			'anchor' => $this->anchor->value,
		);
	}
}
